import Faker from 'faker';
import React from 'react';
import shallowCompare from 'react-addons-shallow-compare';

export default
class ComboBox extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            currentIndex: -1,
            expanded: false,
            value: props.value
        };

        this.listBoxID = Faker.random.uuid();
        this.activeID = Faker.random.uuid();
    }

    shouldComponentUpdate(nextProps, nextState) {
        return shallowCompare(this, nextProps, nextState);
    }

    componentWillReceiveProps(nextProps) {
        this.setState({
            value: nextProps.value
        });
    }

    _onBlur(event) {
        this.tryToBlur = setInterval(() => {
            if (null == this.previousOption) {
                this.setState({
                    currentIndex: -1,
                    expanded: false
                });
                clearInterval(this.tryToBlur);
            }
        }, 100);

        this.props.onBlur(event);
    }

    _onChange(event) {
        this.setState({
            value: event.target.value
        });

        this.props.onChange(event);
    }

    _onButtonClick() {
        this.setState({
            expanded: !this.state.expanded
        });
        clearInterval(this.tryToBlur);

        this.refs.comboBox.focus();
    }

    _onKeyDown(event) {
        if (
            (
                9 !== event.keyCode ||
                -1 !== this.state.currentIndex ||
                -1 !== [8, 13, 38, 40].indexOf(event.keyCode) ||
                46 <= event.keyCode
            ) &&
            -1 === [91, 92, 93].indexOf(event.keyCode) &&
            (111 >= event.keyCode || 146 <= event.keyCode)
        ) {
            switch (event.keyCode) {
                case 40:
                    if (event.target.nextSibling.childNodes.length - 1 > this.state.currentIndex) {
                        this.setState({
                            currentIndex: this.state.currentIndex + 1,
                            expanded: true
                        });
                    }
                    break;
                case 38:
                    if (0 < this.state.currentIndex) {
                        this.setState({
                            currentIndex: this.state.currentIndex - 1
                        });
                    } else if (0 === this.state.currentIndex) {
                        this.setState({
                            currentIndex: this.state.currentIndex - 1,
                            expanded: false
                        });
                    }
                    break;
                case 9:
                    event.preventDefault();
                case 13:
                    this.setState({
                        currentIndex: -1,
                        expanded: false,
                        value: event.target.nextSibling.childNodes[this.state.currentIndex].childNodes[0].getAttribute('value')
                    });
                    break;
                default:
                    this.setState({
                        expanded: true
                    });
            }
        }

        this.props.onKeyDown(event);
    }

    _onOptionMouseDown(event) {
        this.previousOption = event.target.parentNode.getAttribute('tabindex');

        this.setState({
            expanded: true
        });
    }

    _onOptionMouseMove(event) {
        this.setState({
            currentIndex: parseInt(event.target.parentNode.getAttribute('tabindex'))
        });
    }

    _onOptionMouseUp(event) {
        if (this.previousOption === event.target.parentNode.getAttribute('tabindex')) {
            this.setState({
                currentIndex: -1,
                expanded: false,
                value: event.target.getAttribute('value')
            });
        }

        this.previousOption = null;
        clearInterval(this.tryToBlur);
    }

    render() {
        var { applyFilter, children, onBlur, onChange, onKeyDown, value, ...rest } = this.props;

        if (this.state.expanded) {
            rest['aria-expanded'] = true;
        }

        return (
            <span className="reactive-combo-box">
                <input
                    ref="comboBox"
                    type="text"
                    onBlur={this._onBlur.bind(this)}
                    onChange={this._onChange.bind(this)}
                    onKeyDown={this._onKeyDown.bind(this)}
                    value={this.state.value}
                    aria-activedescendant={this.activeID}
                    aria-autocomplete="inline"
                    aria-owns={this.listBoxID}
                    role="combobox"
                    { ...rest }
                />
                <ul role="listbox" id={this.listBoxID}>
                    {applyFilter(children, this.state.value).map((content, index) => {
                        var opts = {};
                        if (content.props.value === this.state.value) {
                            opts['id'] = this.activeID;
                            opts['className'] = 'active';
                        }

                        if (index === this.state.currentIndex) {
                            opts['className'] = ' hover';
                        }

                        return (
                            <li
                                onMouseDown={this._onOptionMouseDown.bind(this)}
                                onMouseMove={this._onOptionMouseMove.bind(this)}
                                onMouseUp={this._onOptionMouseUp.bind(this)}
                                key={content.props.value}
                                role="option"
                                tabIndex={index}
                                {...opts}
                            >
                                {content}
                            </li>
                        );
                    })}
                </ul>
                <button onClick={this._onButtonClick.bind(this)}>▾</button>
            </span>
        );
    }
}

ComboBox.propTypes = {
    applyFilter: React.PropTypes.func.isRequired,
    name: React.PropTypes.string.isRequired,
    onBlur: React.PropTypes.func.isRequired,
    onChange: React.PropTypes.func.isRequired,
    onKeyDown: React.PropTypes.func.isRequired,
    value: React.PropTypes.string.isRequired
};

ComboBox.defaultProps = {
    applyFilter: (children, expected) => {
        return children.filter((content) => {
            return -1 < content.props.value.indexOf(expected);
        });
    },
    onBlur: () => {
    },
    onKeyDown: () => {
    },
    value: ''
};
